@@ -269,14 +269,41 @@
 	   (with sp (path-previous-section bt bp)
 	     (and (!= sp bp) (path->tree (append (tree->path bt) sp))))))))
 
+(define (selection-trim-ending)
+  (if (selection-active-any?)
+    (with st (selection-tree)
+      (if (and (not  (tree-atomic? st ))
+               (tree-empty? (tree-ref st :last)))             
+        (begin 
+          (selection-set 
+            (selection-get-start) 
+            (path-previous (root-tree) (selection-get-end)))
+          (selection-trim-ending))))))
+
+(define (make-section-aux l flag)
+  (if (selection-active-any?) 
+    (let 
+      ((cp (cursor-path))
+       (selstart (selection-get-start)))
+      (selection-trim-ending)
+      (if (tree-multi-paragraph? (selection-tree))
+        (set-message "make-section error" "invalid multi-paragraph selection")
+        (with selend  (selection-get-end)
+          (make l)
+          (if flag (make-return-before)
+            (if (or (path-less-eq? cp selstart) (path-less? selend cp))
+              ;; reposition cursor when its path still exists
+              (go-to cp))))))
+    (if (not (make-return-after))
+      (begin 
+        (make l)
+        (if flag (make-return-before))))))
+       
 (tm-define (make-section l)
-  (if (or (selection-active-any?) (not (make-return-after)))
-      (make l)))
+  (make-section-aux l #f))
 
 (tm-define (make-unnamed-section l)
-  (if (or (selection-active-any?) (not (make-return-after)))
-      (make l)
-      (make-return-before)))
+  (make-section-aux l #t))
 
 (tm-define (kbd-enter t shift?)
   (:require (section-context? t))
@@ -352,6 +379,18 @@
   (:require (in? l (list "itemize-levels" "enumerate-levels")))
   (list "1" "2" "3" "4"))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Hyphenation
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-define (hyphenate-selection-as hyph)
+  (:interactive #t)
+  (:argument hyph "Hyphenate as")
+  (:proposals hyph (list (tm->string (selection-tree))))
+  (with ins `(hyphenate-as ,hyph ,(selection-tree))
+    (clipboard-cut "null")
+    (insert ins)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Inserting formulas
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -384,14 +423,25 @@
   (if (not (make-return-after))
       (insert (list (string->symbol env) aux name '(document "")))))
 
-(tm-define (make-bib style file-name)
-  (:argument style "Bibliography style")
-  (:proposals style '("tm-plain" "tm-alpha" "tm-acm" "tm-elsart-num"
-                      "tm-ieeetr" "tm-siam" "tm-unsrt"))
+(define (normalized-bib-name f)
+  (if (string? f) f
+      (let* ((r (url-delta (current-buffer) f))
+             (n (if (== (url-suffix r) "bib") (url-unglue r 4) r)))
+        (url->string n))))
+
+(tm-define (make-bib file-name)
   (:argument file-name "Bibliography file")
+  (let* ((name (normalized-bib-name file-name))
+         (aux (if (context-has? "bib-prefix") (get-env "bib-prefix") "bib")))
+    (if (not (make-return-after))
+        (insert-go-to `(bibliography ,aux "tm-plain" ,name (document ""))
+                      '(3 0 0)))))
+
+(tm-define (make-database-bib)
   (with aux (if (context-has? "bib-prefix") (get-env "bib-prefix") "bib")
     (if (not (make-return-after))
-        (insert (list 'bibliography aux style file-name '(document ""))))))
+        (insert-go-to `(bibliography ,aux "tm-plain" "" (document ""))
+                      '(3 0 0)))))
 
 (tm-define (automatic-section-context? t)
   (tree-in? t (automatic-section-tag-list)))
@@ -668,45 +718,118 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (tm-define (float-context? t)
-  (cond ((tree-is? t 'float) #t)
+  (tree-in? t '(float wide-float)))
+
+(tm-define (footnote-context? t)
+  (tree-in? t '(footnote wide-footnote)))
+
+(tm-define (float-or-footnote-context? t)
+  (tree-in? t '(float wide-float footnote wide-footnote)))
+
+(tm-define (phantom-float-context? t)
+  (tree-is? t 'phantom-float))
+
+(tm-define (rich-float-context? t)
+  (cond ((tree-in? t '(float wide-float)) #t)
         ((tree-in? t '(big-figure big-figure*
                        big-table big-table*
                        algorithm algorithm*
                        document concat))
-         (and (tree-up t) (float-context? (tree-up t))))
+         (and (tree-up t) (rich-float-context? (tree-up t))))
         (else #f)))
 
 (tm-define (floatable-context? t)
   (and (tree-in? t '(big-figure big-figure*
                      big-table big-table*
                      algorithm algorithm*))
-       (not (tree-innermost t 'float))
-       (tree-is? t :up 'document)))
+       (not (rich-float-context? t))
+       (or (tree-is? t :up 'document)
+           (and (tree-is? t :up 'with)
+                (tree-is? t :up :up 'document)))))
+
+(define (with-wide? w)
+  (and (tree-is? w 'with)
+       (== (tree-arity w) 3)
+       (tm-equal? (tree-ref w 0) "par-columns")
+       (tm-equal? (tree-ref w 1) "1")))
 
 (tm-define (turn-floating t)
   (when (floatable-context? t)
-    (with f `(float "float" "thb" ,t)
+    (when (tree-is? t :up 'with)
+      (set! t (tree-up t)))
+    (with f (if (with-wide? t)
+                `(wide-float "float" "thb" ,(tree-ref t 2))
+                `(float "float" "thb" ,t))
       (tree-set! t f)
       (tree-go-to t :start)
-      (remove-text #f))))
+      (remove-text #f)
+      (tree-go-to t :end))))
+
+(define (floatable-path t)
+  (if (or (tm-func? t 'document 1)
+          (tm-func? t 'with 3)
+          (tm-in? t '(big-figure big-figure*
+                      big-table big-table*
+                      algorithm algorithm*)))
+      (with last (- (tm-arity t) 1)
+        (cons last (floatable-path (tm-ref t :last))))
+      (path-end t '())))
+
+(tm-define (turn-non-floating t)
+  (when (float-context? t)
+    (with body (tree-copy (tree-ref t :last))
+      (when (tree-is? t 'wide-float)
+        (set! body (tm->tree `(with "par-columns" "1" ,body))))
+      (when (not (tree-is? body 'document))
+        (set! body (tm->tree `(document ,body))))
+      (tree-cut t)
+      (insert-return)
+      (insert-go-to body (floatable-path body)))))
+
+(tm-define (float-wide? t)
+  (and-with f (tree-search-upwards t float-or-footnote-context?)
+    (tree-in? f '(wide-float wide-footnote))))
+
+(tm-define (test-float-wide? . args)
+  (float-wide? (focus-tree)))
+(tm-define (float-toggle-wide t)
+  (:check-mark "v" test-float-wide?)
+  (and-with f (tree-search-upwards t float-or-footnote-context?)
+    (cond ((tree-is? f 'float) (tree-assign-node f 'wide-float))
+          ((tree-is? f 'wide-float) (tree-assign-node f 'float))
+          ((tree-is? f 'footnote) (tree-assign-node f 'wide-footnote))
+          ((tree-is? f 'wide-footnote) (tree-assign-node f 'footnote)))))
+
+(tm-define (floatable-wide? t)
+  (and-with w (tree-up t)
+    (with-wide? w)))
+
+(tm-define (test-floatable-wide? . args)
+  (floatable-wide? (focus-tree)))
+(tm-define (floatable-toggle-wide t)
+  (:check-mark "v" test-floatable-wide?)
+  (and-with w (tree-up t)
+    (if (with-wide? w)
+        (tree-set w (tree-ref w 2))
+        (tree-set t `(with "par-columns" "1" ,t)))))
 
 (tm-define (cursor-at-anchor?)
   (with t (cursor-tree)
-    (tree-in? t '(float footnote))))
+    (float-or-footnote-context? t)))
 
 (tm-define (go-to-anchor)
-  (cond ((inside? 'float)
-         (with-innermost t 'float
+  (cond ((or (inside? 'float) (inside? 'wide-float))
+         (with-innermost t float-context?
            (tree-go-to t :end)))
-        ((inside? 'footnote)
-         (with-innermost t 'footnote
+        ((or (inside? 'footnote) (inside? 'wide-footnote))
+         (with-innermost t footnote-context?
            (tree-go-to t :end)))))
 
 (tm-define (go-to-float)
   (with t (cursor-tree)
-    (cond ((tree-is? t 'float)
+    (cond ((float-context? t)
            (tree-go-to t 2 :start))
-          ((tree-is? t 'footnote)
+          ((footnote-context? t)
            (tree-go-to t 0 :start)))))
 
 (tm-define (cursor-toggle-anchor)
